
-- +migrate Up

CREATE EXTENSION IF NOT EXISTS "uuid-ossp";

CREATE TYPE "cluster_status" AS ENUM (
    'PENDING_CREATE',
    'ACTIVE',
    'PENDING_UPDATE',
    'PENDING_UPGRADE',
    'PENDING_ROTATE_CERTS',
    'PENDING_DELETE',
    'PENDING_RESIZE',
    'PENDING_NODE_REINSTALL',
    'ERROR'
    );

CREATE TYPE "task_status" AS ENUM (
    'IN_PROGRESS',
    'DONE',
    'ERROR'
    );

CREATE TYPE "task_type" AS ENUM (
    'CREATE_CLUSTER',
    'DELETE_CLUSTER',
    'ROTATE_CERTS',
    'NODE_GROUP_RESIZE',
    'NODE_REINSTALL',
    'CLUSTER_RESIZE'
    );

CREATE TABLE "clusters" (
    "id" uuid PRIMARY KEY DEFAULT (uuid_generate_v4()),
    "created_at" timestamp DEFAULT (now()),
    "updated_at" timestamp,
    "deleted_at" timestamp,
    "name" varchar(255) NOT NULL,
    "status" "cluster_status" DEFAULT 'PENDING_CREATE',
    "selectel_client_id" varchar(255) NOT NULL,
    "region" varchar(255) NOT NULL,
    "service_network_id" uuid,
    "service_subnet_id" uuid,
    "end_user_network_id" uuid,
    "end_user_subnet_id" uuid,
    "external_network_id" uuid,
    "router_id" uuid,
    "rbac_id" uuid,
    "using_existing_network_and_subnet" bool,
    "kube_api_load_balancer_id" uuid,
    "kube_api_floating_ip_id" uuid,
    "kube_api_floating_ip" varchar(21),
    "project_id" uuid NOT NULL,
    "project_token" varchar(255) NOT NULL,
    "user_id" uuid NOT NULL,
    "kube_version" varchar(255),
    "trustee_user_id" uuid,
    "trustee_user_name" varchar(255),
    "trustee_user_password" varchar(255),
    "trust_id" uuid,
    "additional_params" jsonb
);

CREATE TABLE "kube_versions" (
    "version" varchar(255) PRIMARY KEY,
    "created_at" timestamp DEFAULT (now()),
    "updated_at" timestamp,
    "deleted_at" timestamp,
    "is_default" boolean,
    "is_supported" boolean
);

CREATE TABLE "servers" (
    "id" uuid PRIMARY KEY DEFAULT (uuid_generate_v4()),
    "created_at" timestamp DEFAULT (now()),
    "updated_at" timestamp,
    "deleted_at" timestamp,
    "hostname" varchar(255) NOT NULL,
    "main_ip" varchar(21) NOT NULL,
    "secondary_ip" varchar(21),
    "volume_id" uuid,
    "os_server_id" uuid NOT NULL,
    "group_id" uuid NOT NULL
);

CREATE TABLE "server_groups" (
    "id" uuid PRIMARY KEY DEFAULT (uuid_generate_v4()),
    "created_at" timestamp DEFAULT (now()),
    "updated_at" timestamp,
    "deleted_at" timestamp,
    "cluster_id" uuid NOT NULL,
    "flavor_id" uuid NOT NULL,
    "volume_gb" integer NOT NULL,
    "volume_type" varchar(255),
    "local_volume" bool,
    "image_id" uuid NOT NULL,
    "key_name" varchar(255),
    "is_service_project" bool,
    "os_server_group_id" uuid NOT NULL,
    "availability_zone" varchar(255) NOT NULL,
    "ssh_key_private" bytea,
    "ssh_key_public" bytea
);

CREATE TABLE "pki_trees" (
    "id" uuid PRIMARY KEY DEFAULT (uuid_generate_v4()),
    "created_at" timestamp DEFAULT (now()),
    "updated_at" timestamp,
    "deleted_at" timestamp,
    "cluster_id" uuid NOT NULL,
    "root_ca_cert" bytea NOT NULL,
    "root_ca_key" bytea NOT NULL,
    "etcd_ca_cert" bytea NOT NULL,
    "etcd_ca_key" bytea NOT NULL,
    "front_proxy_ca_cert" bytea NOT NULL,
    "front_proxy_ca_key" bytea NOT NULL,
    "admin_cert" bytea NOT NULL,
    "admin_key" bytea NOT NULL,
    "certs_duration_days" int NOT NULL
);

CREATE TABLE "tasks" (
    "id" uuid PRIMARY KEY DEFAULT (uuid_generate_v4()),
    "started_at" timestamp DEFAULT (now()),
    "updated_at" timestamp,
    "status" "task_status" DEFAULT 'IN_PROGRESS',
    "cluster_id" uuid NOT NULL,
    "type" "task_type" NOT NULL
);

ALTER TABLE "clusters" ADD FOREIGN KEY ("kube_version") REFERENCES "kube_versions"("version");

ALTER TABLE "servers" ADD FOREIGN KEY ("group_id") REFERENCES "server_groups" ("id");

ALTER TABLE "server_groups" ADD FOREIGN KEY ("cluster_id") REFERENCES "clusters" ("id");

ALTER TABLE "pki_trees" ADD FOREIGN KEY ("cluster_id") REFERENCES "clusters" ("id");

ALTER TABLE "tasks" ADD FOREIGN KEY ("cluster_id") REFERENCES "clusters" ("id");

-- clusters indexes
CREATE INDEX ON "clusters" ("created_at");
CREATE INDEX ON "clusters" ("deleted_at");
CREATE INDEX ON "clusters" ("project_id");
CREATE INDEX ON "clusters" ("kube_version");
CREATE INDEX ON "clusters" ("project_id");
CREATE INDEX ON "clusters" ("selectel_client_id");

-- servers indexes
CREATE INDEX ON "servers" ("created_at");
CREATE INDEX ON "servers" ("deleted_at");
CREATE INDEX ON "servers" ("group_id");

-- server_groups indexes
CREATE INDEX ON "server_groups" ("created_at");
CREATE INDEX ON "server_groups" ("deleted_at");
CREATE INDEX ON "server_groups" ("cluster_id");

-- pki_trees indexes
CREATE INDEX ON "pki_trees" ("created_at");
CREATE INDEX ON "pki_trees" ("deleted_at");
CREATE UNIQUE INDEX ON "pki_trees" ("cluster_id");

-- tasks indexes
CREATE INDEX ON "tasks" ("started_at");
CREATE INDEX ON "tasks" ("updated_at");
CREATE INDEX ON "tasks" ("cluster_id");

-- kube_versions indexes
CREATE INDEX ON "kube_versions" ("created_at");
CREATE INDEX ON "kube_versions" ("updated_at");
CREATE INDEX ON "kube_versions" ("deleted_at");
CREATE UNIQUE INDEX ON "kube_versions" ("is_default") WHERE "is_default" = true;

-- +migrate Down

DROP TABLE clusters, kube_versions, server_groups, servers, pki_trees, tasks CASCADE;

DROP TYPE cluster_status, task_status, task_type CASCADE;

DROP EXTENSION "uuid-ossp";
